#include <fcntl.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/mman.h>
#include <unistd.h>

#define SEGMENT_NAME "/my_test_segment"

int
main(int argc, char *argv[])
{
	int loops, i;
	off_t size;
	bool fallocatep;
	bool memsetp;
	bool hugep;
	void *mem;

	if (argc != 6)
	{
		fprintf(stderr, "Usage: %s <loops> <size_in_mb> <fallocate?> <memset?> <huge?>\n", argv[0]);
		return EXIT_FAILURE;
	}

	loops = atoi(argv[1]);
	size = atoi(argv[2]) * 1024 * 1024;
	fallocatep = atoi(argv[3]) != 0;
	memsetp = atoi(argv[4]) != 0;
	hugep = atoi(argv[5]) != 0;

	for (i = 0; i < loops; ++i)
	{
		int fd;

		fd = shm_open(SEGMENT_NAME, O_CREAT | O_RDWR, S_IWUSR | S_IRUSR);
		if (fd < 0)
		{
			perror("shm_open");
			goto cleanup;
		}
		if (ftruncate(fd, size))
		{
			perror("ftruncate");
			goto cleanup;
		}
		if (fallocatep && fallocate(fd, 0, 0, size))
		{
			perror("fallocate");
			goto cleanup;
		}
		mem = mmap(0, size, PROT_READ | PROT_WRITE, MAP_SHARED | (hugep ? MAP_HUGETLB : 0), fd, 0);
		if (mem == NULL)
		{
			fprintf(stderr, "mmap failed");
			goto cleanup;
		}
		if (memsetp)
			memset(mem, 0, size);
		munmap(mem, size);
		close(fd);
	}

	shm_unlink(SEGMENT_NAME);
	return EXIT_SUCCESS;

cleanup:	
	shm_unlink(SEGMENT_NAME);
	return EXIT_FAILURE;
}
